"""Test ferda context processors."""
from unittest.mock import patch

from django.contrib.auth.models import AnonymousUser, Permission
from django.test import RequestFactory, TestCase, override_settings
from django.urls import reverse

from ferda.context_processors import django_settings, ferda_version, user_permissions, user_profile
from ferda.models import User
from ferda.plugins import PluginConfig


class TestFerdaModules(TestCase):
    """Test ferda_modules."""

    @override_settings(ROOT_URLCONF='ferda.tests.urls', DJANGO_FIDO_TWO_STEP_AUTH=False)
    @patch('ferda.context_processors.CONFIG', {})
    def test_empty(self):
        response = self.client.get(reverse('registry:login'))
        self.assertEqual(response.context['ferda_plugins'], {})

    @override_settings(ROOT_URLCONF='ferda.tests.urls', DJANGO_FIDO_TWO_STEP_AUTH=False)
    @patch(
        'ferda.context_processors.CONFIG',
        {'registry': PluginConfig(name='Registry', url='/', access=lambda x: True)}
    )
    def test_access_true(self):
        response = self.client.get(reverse('registry:login'))
        self.assertEqual(response.context['ferda_plugins'], {
            'registry': {
                'name': 'Registry',
                'url': '/',
                'icon': None,
            }
        })

    @override_settings(ROOT_URLCONF='ferda.tests.urls', DJANGO_FIDO_TWO_STEP_AUTH=False)
    @patch(
        'ferda.context_processors.CONFIG',
        {'registry': PluginConfig(name='Registry', url='/', access=lambda x: False)}
    )
    def test_access_false(self):
        response = self.client.get(reverse('registry:login'))
        self.assertEqual(response.context['ferda_plugins'], {})


class TestUserProfile(TestCase):
    """Test user_profile."""

    def setUp(self):
        self.user = User.objects.create_user(
            'user', 'user@example.com', 'password', first_name='Harry', last_name='Potter')
        self.request_factory = RequestFactory()

    def test_authenticated(self):
        request = self.request_factory.get('/', {})
        request.user = self.user
        self.assertEqual(user_profile(request), {
            'user_profile': {
                'username': 'user',
                'first_name': 'Harry',
                'last_name': 'Potter',
                'email': 'user@example.com',
                'is_staff': False,
            }
        })

    def test_anonymous(self):
        request = self.request_factory.get('/', {})
        request.user = AnonymousUser()
        self.assertEqual(user_profile(request), {})


class TestUserPermissions(TestCase):
    """Test user_permissions."""

    def setUp(self):
        self.user = User.objects.create_user(
            'user', 'user@example.com', 'password', first_name='Luke', last_name='Skywalker')
        self.user.user_permissions.set(
            Permission.objects.filter(codename__in=('can_view_registry', 'can_edit_registrar')))
        self.request_factory = RequestFactory()

    def test_authenticated(self):
        request = self.request_factory.get('/', {})
        request.user = self.user
        self.assertEqual(
            set(user_permissions(request)['user_permissions']),
            set(['ferda.can_view_registry', 'ferda.can_edit_registrar']),
        )

    def test_anonymous(self):
        request = self.request_factory.get('/', {})
        request.user = AnonymousUser()
        self.assertEqual(user_permissions(request), {})


class TestDjangoSettings(TestCase):
    """Test django_settings."""

    def setUp(self):
        self.request_factory = RequestFactory()
        self.request = self.request_factory.get('/', {})

    @override_settings(
        TIME_ZONE='Europe/Prague',
        LANGUAGE_CODE='cs',
        LANGUAGES=(('en', 'English'), ('cs', 'Czech')),
        AUTHENTICATION_BACKENDS=['django.contrib.auth.backends.ModelBackend'],
        FERDA_MANUAL_IN_ZONE_DEFAULT_DURATION=3*24*60*60,
        FERDA_OBJECT_DETAIL_LOGGER_SERVICES=['Service 1', 'Service 2'],
    )
    def test_settings(self):
        self.assertEqual(django_settings(self.request), {
            'django_settings': {
                'TIME_ZONE': 'Europe/Prague',
                'LANGUAGE_CODE': 'cs',
                'LANGUAGES': (('en', 'English'), ('cs', 'Czech')),
                'FIDO2_AUTHENTICATION': False,
                'MANUAL_IN_ZONE_DEFAULT_DURATION': 3*24*60*60,
                'OBJECT_DETAIL_LOGGER_SERVICES': ['Service 1', 'Service 2'],
            }
        })


class TestFerdaVersion(TestCase):
    """Test ferda_version."""

    def setUp(self):
        self.request_factory = RequestFactory()
        self.request = self.request_factory.get('/', {})

    @patch('ferda.__version__', '0.1.2rc3+test')
    def test_version(self):
        self.assertEqual(ferda_version(self.request), {
            'ferda_version': {
                'version': '0.1.2rc3+test',
            }
        })
